;;; DO NOT MODIFY THIS FILE
(if (featurep 'riece-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'riece :version 1.23 :author-version "3.1.2" :type 'regular)

;;;***

;;;### (autoloads (riece-async-open-network-stream) "riece-async" "lisp/riece-async.el")

(autoload 'riece-async-open-network-stream "riece-async" nil nil nil)

;;;***

;;;### (autoloads (riece) "riece" "lisp/riece.el")

(autoload 'riece "riece" "\
Connect to the IRC server and start chatting.
If optional argument CONFIRM is non-nil, ask which IRC server to connect." t nil)

;;;***

(provide 'riece-autoloads)
